{%if optionsDialogBody is not none and optionsDialogBody|length > 0 %}
	{%include module + ":chartHeader.html"%}
	<br><br>
{%endif%}
<span style='color: red; font-weight: bold'>{{errorMessage}}</span>
<br><br>